#include "iodefine.h"
#include "display.h"

#define delayms(a)		delay_ms(a)

#define DDR_REST	PORT5.PDR.BIT.B1
#define DR_REST		PORT5.PODR.BIT.B1
#define LCD_nRESET	DR_REST

static
void LCD_CtrlWrite_ILI9325(volatile unsigned short index, volatile unsigned short data)
{
	*(volatile unsigned char*)0x05000000 = index >> 8;
	*(volatile unsigned char*)0x05000000 = index;

	*(volatile unsigned char*)0x05020000 = data >> 8;
	*(volatile unsigned char*)0x05020000 = data;
	return;
}

static
void LCD_IndexWrite_ILI9325(volatile unsigned short index)
{
	*(volatile unsigned char*)0x05000000 = index >> 8;
	*(volatile unsigned char*)0x05000000 = index;
	return;
}

static
void LCD_DataWrite_ILI9325(volatile unsigned short data)
{
	*(volatile unsigned char*)0x05020000 = data >> 8;
	*(volatile unsigned char*)0x05020000 = data;
	return;
}

static
unsigned short LCD_DataRead_ILI9325(void)
{
	return *(volatile unsigned short*)0x05000002;
}

static
void ILI9325_Initial(void) 
{ 
// VCI=2.8V
//************* Reset LCD Driver ****************//
LCD_nRESET = 1;
delayms(1); // Delay 1ms
LCD_nRESET = 0;
delayms(10); // Delay 10ms // This delay time is necessary
LCD_nRESET = 1;
delayms(50); // Delay 50 ms
//************* Start Initial Sequence **********//
LCD_CtrlWrite_ILI9325(0x0001, 0x0100); // set SS and SM bit
LCD_CtrlWrite_ILI9325(0x0002, 0x0700); // set 1 line inversion
//LCD_CtrlWrite_ILI9325(0x0003, 0x1030); // set GRAM write direction and BGR=1.
LCD_CtrlWrite_ILI9325(0x0003, 0x1018); // set GRAM write direction and BGR=1.
LCD_CtrlWrite_ILI9325(0x0004, 0x0000); // Resize register
LCD_CtrlWrite_ILI9325(0x0008, 0x0207); // set the back porch and front porch
LCD_CtrlWrite_ILI9325(0x0009, 0x0000); // set non-display area refresh cycle ISC[3:0]
LCD_CtrlWrite_ILI9325(0x000A, 0x0000); // FMARK function
LCD_CtrlWrite_ILI9325(0x000C, 0x0000); // RGB interface setting
LCD_CtrlWrite_ILI9325(0x000D, 0x0000); // Frame marker Position
LCD_CtrlWrite_ILI9325(0x000F, 0x0000); // RGB interface polarity
//*************Power On sequence ****************//
LCD_CtrlWrite_ILI9325(0x0010, 0x0000); // SAP, BT[3:0], AP, DSTB, SLP, STB
LCD_CtrlWrite_ILI9325(0x0011, 0x0007); // DC1[2:0], DC0[2:0], VC[2:0]
LCD_CtrlWrite_ILI9325(0x0012, 0x0000); // VREG1OUT voltage
LCD_CtrlWrite_ILI9325(0x0013, 0x0000); // VDV[4:0] for VCOM amplitude
delayms(200); // Dis-charge capacitor power voltage
LCD_CtrlWrite_ILI9325(0x0010, 0x1490); // SAP, BT[3:0], AP, DSTB, SLP, STB
LCD_CtrlWrite_ILI9325(0x0011, 0x0227); // Set DC1[2:0], DC0[2:0], VC[2:0]
delayms(50); // Delay 50ms
LCD_CtrlWrite_ILI9325(0x0012, 0x001A); // External reference voltage= Vci;
delayms(50); // Delay 50ms
LCD_CtrlWrite_ILI9325(0x0013, 0x1400); // VDV[4:0] for VCOM amplitude
LCD_CtrlWrite_ILI9325(0x0029, 0x0019); // VCM[5:0] for VCOMH
LCD_CtrlWrite_ILI9325(0x002B, 0x000B); // Set Frame Rate
delayms(50); // Delay 50ms
LCD_CtrlWrite_ILI9325(0x0020, 0x0000); // GRAM horizontal Address
LCD_CtrlWrite_ILI9325(0x0021, 0x0000); // GRAM Vertical Address
// ----------- Adjust the Gamma Curve ----------//
LCD_CtrlWrite_ILI9325(0x0030, 0x0000);
LCD_CtrlWrite_ILI9325(0x0031, 0x0607);
LCD_CtrlWrite_ILI9325(0x0032, 0x0305);
LCD_CtrlWrite_ILI9325(0x0035, 0x0000);
LCD_CtrlWrite_ILI9325(0x0036, 0x1604);
LCD_CtrlWrite_ILI9325(0x0037, 0x0204);
LCD_CtrlWrite_ILI9325(0x0038, 0x0001);
LCD_CtrlWrite_ILI9325(0x0039, 0x0707);
LCD_CtrlWrite_ILI9325(0x003C, 0x0000);
LCD_CtrlWrite_ILI9325(0x003D, 0x000F);
//------------------ Set GRAM area ---------------//
LCD_CtrlWrite_ILI9325(0x0050, 0x0000); // Horizontal GRAM Start Address
LCD_CtrlWrite_ILI9325(0x0051, 0x00EF); // Horizontal GRAM End Address
LCD_CtrlWrite_ILI9325(0x0052, 0x0000); // Vertical GRAM Start Address
LCD_CtrlWrite_ILI9325(0x0053, 0x013F); // Vertical GRAM Start Address
LCD_CtrlWrite_ILI9325(0x0060, 0xA700); // Gate Scan Line
LCD_CtrlWrite_ILI9325(0x0061, 0x0001); // NDL,VLE, REV
LCD_CtrlWrite_ILI9325(0x006A, 0x0000); // set scrolling line
//-------------- Partial Display Control ---------//
LCD_CtrlWrite_ILI9325(0x0080, 0x0000);
LCD_CtrlWrite_ILI9325(0x0081, 0x0000);
LCD_CtrlWrite_ILI9325(0x0082, 0x0000);
LCD_CtrlWrite_ILI9325(0x0083, 0x0000);
LCD_CtrlWrite_ILI9325(0x0084, 0x0000);
LCD_CtrlWrite_ILI9325(0x0085, 0x0000);
//-------------- Panel Control -------------------//
LCD_CtrlWrite_ILI9325(0x0090, 0x0010);
LCD_CtrlWrite_ILI9325(0x0092, 0x0600);
LCD_CtrlWrite_ILI9325(0x0093, 0x0003);
LCD_CtrlWrite_ILI9325(0x0095, 0x0110);
LCD_CtrlWrite_ILI9325(0x0097, 0x0000);
LCD_CtrlWrite_ILI9325(0x0098, 0x0000);
LCD_CtrlWrite_ILI9325(0x0007, 0x0133); // 262K color and display ON
}

void init_display(void)
{
	SYSTEM.SYSCR0.WORD = 0x5a03;

	MPC.PWPR.BIT.B0WI = 0;
	MPC.PWPR.BIT.PFSWE = 1;

	MPC.PFCSE.BIT.CS3E = 1;
	MPC.PFCSS0.BIT.CS3S = 2;
	MPC.PFAOE1.BIT.A17E = 1;
	
	BSC.CS3CR.WORD = 0x0001 | (2 << 4);
	BSC.CS3MOD.WORD = 0x8001;
	BSC.CS3WCR1.LONG = 0x03030303;
	BSC.CS3WCR2.LONG = 0x11110111;
	BSC.CS3REC.WORD = 0x0000;

	DR_REST		= 1;
	DDR_REST	= 1;
	delayms(10);
	ILI9325_Initial();
	ILI9325_Initial();

	LCD_IndexWrite_ILI9325(0x0022);

#if 0
	unsigned int i,n,m;
	unsigned short color[8]={0xf800,0x07e0,0x001f,0x07ff,0xf81f,0xffe0,0x0000,0xffff};

	while(1)
	{
		for(n=0;n<8;n++)
			for(i=0;i<40;i++)
				for(m=0;m<240;m++)
					LCD_DataWrite_ILI9325(0xffff);

		for(n=0;n<8;n++)
			for(i=0;i<40;i++)
				for(m=0;m<240;m++)
					LCD_DataWrite_ILI9325(color[n]);
	}
#endif
}

static
void area_set(int xs, int ys, int xe, int ye, int x, int y)
{
    LCD_CtrlWrite_ILI9325(0x0050, xs);	// Horizontal GRAM Start Address 
    LCD_CtrlWrite_ILI9325(0x0051, xe);	// Horizontal GRAM End Address 
    LCD_CtrlWrite_ILI9325(0x0052, ys);	// Vertical GRAM Start Address 
    LCD_CtrlWrite_ILI9325(0x0053, ye);	// Vertical GRAM End Address 

    LCD_CtrlWrite_ILI9325(0x0020, x);	// GRAM horizontal Address 
    LCD_CtrlWrite_ILI9325(0x0021, y);	// GRAM Vertical Address  

	return;
}

void display_area_set(int xs, int ys, int xe, int ye)
{
	area_set(ys, xs, ye, xe, ye, xs);
	LCD_IndexWrite_ILI9325(0x0022);
}

void display_write_dot(int x, int y, unsigned short color)
{
	display_area_set(319 - x, y, 319 - x, y);

	LCD_DataWrite_ILI9325(color);
	return;
}

unsigned short display_read_dot(int x, int y)
{
	display_area_set(319 - x, y, 319 - x, y);

	return  LCD_DataRead_ILI9325();
}

void display_write_rect(int x1, int y1, int x2, int y2, unsigned short color)
{
	int lx = x2 - x1 + 1;
	int ly = y2 - y1 + 1;

	display_area_set(319 - x2, y1, 319 - x1, y2);

	int n;
	for(n = 0; n < lx * ly; n++)
	{
		LCD_DataWrite_ILI9325(color);
	}
	return;
}

void display_block_copy(int sx, int sy, int lx, int ly, unsigned short *buf)
{
	display_area_set(319 - (sx + lx - 1), sy, 319 - sx, sy + ly - 1);

	int n;
	for(n = 0; n < lx * ly; n++)
	{
		LCD_DataWrite_ILI9325(*buf++);
	}
	return;
}

void display_block_copy_r(int sx, int sy, int lx, int ly, unsigned short *buf)
{
	LCD_CtrlWrite_ILI9325(0x0003, 0x1008);
	display_block_copy(sx, sy, lx, ly, buf);
	LCD_CtrlWrite_ILI9325(0x0003, 0x1018);
	return;
}
